from django.contrib import admin

from django_dragonpay.models import *


def readable_amount(obj):
    return "{} {:,}".format(obj.currency, obj.amount)


class StatusPayoutsFilter(admin.SimpleListFilter):
    '''Admin filter for Payouts Status'''
    title = 'Status'
    parameter_name = 'status'

    def lookups(self, request, model_admin):
        return model_admin.model.STATUS_CODES

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(status=self.value())


class PayoutUserAdmin(admin.ModelAdmin):
    search_fields = ('first_name', 'last_name', 'email')


class TransactionAdmin(admin.ModelAdmin):
    date_heirarchy = 'created_at'
    search_fields = ('txn_id', 'email')
    list_filter = (StatusPayoutsFilter, )
    list_display = ('token', 'status', readable_amount, 'description', 'email')


class PayoutsAdmin(admin.ModelAdmin):
    class CompletedPayoutsFilter(admin.SimpleListFilter):
        '''Admin filter for Completed Payouts.'''
        title = 'Completed'
        parameter_name = 'completed'

        def lookups(self, request, model_admin):
            return (
                ('yes', 'Completed'),
                ('no', 'Not yet completed'),
            )

        def queryset(self, request, queryset):
            if self.value() == 'yes':
                # filter for Success/Failed/Voided
                return queryset.filter(status__in=['S', 'F', 'V'])
            elif self.value() == 'no':
                # filter for Pending/In progress
                return queryset.filter(status__in=['P', 'G'])

    date_heirarchy = 'created_at'
    search_fields = ('txn_id', 'user_id', 'user_name', 'email')
    list_filter = (CompletedPayoutsFilter, StatusPayoutsFilter)
    list_display = (
        'txn_id', 'user_name', 'email', readable_amount, 'description')


admin.site.register(DragonpayPayoutUser, PayoutUserAdmin)
admin.site.register(DragonpayTransaction, TransactionAdmin)
admin.site.register(DragonpayPayout, PayoutsAdmin)



import logging
from django.db import models
from django_dragonpay.settings import DRAGONPAY_SAVE_DATA

__all__ = ['DragonpayPayoutUser', 'DragonpayTransaction', 'DragonpayPayout']

logger = logging.getLogger('dragonpay.models')


class DragonpayPayoutUser(models.Model):
    first_name = models.CharField(max_length=32)
    middle_name = models.CharField(max_length=32)
    last_name = models.CharField(max_length=32)
    email = models.CharField(max_length=64)
    birthdate = models.DateField()
    mobile = models.CharField(max_length=24)
    address1 = models.CharField(max_length=32)
    address2 = models.CharField(max_length=32)
    city = models.CharField(max_length=32)
    country = models.CharField(max_length=16)
    state = models.CharField(max_length=16)
    zip = models.CharField(max_length=8)

    class Meta:
        app_label = 'django_dragonpay'
        verbose_name = "Payout User"
        verbose_name_plural = "Payout Users"


class DragonpayTransaction(models.Model):
    STATUS_CODES = (
        ('S', 'Success'),
        ('F', 'Failed'),
        ('P', 'Pending'),
        ('U', 'Unknown'),
        ('R', 'Refund'),
        ('K', 'Chargeback'),
        ('V', 'Voided'),
        ('A', 'Authorized'),
    )

    CURRENCIES = (('PHP', 'Philippine Peso'), ('USD', 'US Dollar'))

    id = models.CharField(
        primary_key=True, max_length=40, verbose_name='Transaction ID')
    token = models.CharField(max_length=40)                 # tokenid id
    amount = models.DecimalField(max_digits=8, decimal_places=2)
    currency = models.CharField(
        max_length=3, choices=CURRENCIES, default='PHP')
    description = models.CharField(max_length=128)
    email = models.CharField(max_length=40)
    param1 = models.CharField(max_length=80, null=True, blank=True)
    param2 = models.CharField(max_length=80, null=True, blank=True)

    status = models.CharField(max_length=1, choices=STATUS_CODES, default='P')
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = 'django_dragonpay'
        verbose_name = "Transaction"
        verbose_name_plural = "Transactions"

    def __unicode__(self):
        return '%s [%s:%s] %s' % (
            self.get_status_display(), self.id, self.amount, self.email)

    def fetch_status(self):
        '''Query DragonPay for the status of this transaction and update the
        database.'''

        from django_dragonpay.api.soap import get_txn_status
        from django_dragonpay.constants import DRAGONPAY_STATUS_CODES

        status = get_txn_status(self.id)

        if status != self.status:
            logger.info(
                '[%s] status updated to %s',
                self.txn_id, DRAGONPAY_STATUS_CODES[status])

            self.status = status
            self.save(update_fields=['status'])

    @classmethod
    def create_from_dict(cls, details):
        # Check if we should save to database
        if not DRAGONPAY_SAVE_DATA:
            return

        return DragonpayTransaction.objects.create(
            id=details['txn_id'],
            amount=details['amount'],
            currency=details.get('currency') or 'PHP',
            description=details['description'],
            email=details['email'],
            param1=details.get('param1'),
            param2=details.get('param2'),
            token=details.get('token')
        )


class DragonpayPayout(models.Model):
    STATUS_CODES = (
        ('S', 'Success'),
        ('F', 'Failed'),
        ('P', 'Pending'),
        ('G', 'In progress'),
        ('V', 'Voided'),
    )

    PROCESSORS = (
        (('BDO', 'Banco De Oro')),
        (('BPI', 'Bank of the Philippine Islands')),
        (('CBC', 'Chinabank')),
        (('EWB', 'East West Bank')),
        (('LBP', 'Land Bank of the Philippines')),
        (('MBTC'), ('Metrobank')),
        (('PNB'), ('Philippine National Bank')),
        (('RCBC'), ('RCBC')),
        (('SBC'), ('Security Bank')),
        (('UBP'), ('Union Bank')),
        (('UCPB'), ('UCPB')),
        (('PSB'), ('PS Bank')),
        (('CEBL'), ('Cebuana Lhuilier')),
        (('GCSH'), ('GCash')),
        (('SMRT'), ('Smart Money')),
    )

    txn_id = models.CharField(max_length=40)

    # For payout registerd user
    user_id = models.CharField(max_length=40, null=True, blank=True)

    # For non-registered, one time payout
    user_name = models.CharField(max_length=32, null=True, blank=True)
    processor_id = models.CharField(
        max_length=8, null=True, blank=True, choices=PROCESSORS)
    processor_detail = models.CharField(max_length=32, null=True, blank=True)
    email = models.CharField(max_length=32, null=True, blank=True)
    mobile = models.CharField(max_length=32, null=True, blank=True)

    # payout details
    amount = models.DecimalField(max_digits=8, decimal_places=2)
    description = models.CharField(max_length=128)
    currency = models.CharField(max_length=3, default='PHP')
    status = models.CharField(max_length=1, choices=STATUS_CODES, default='P')

    created_at = models.DateTimeField(auto_now_add=True)    # timestamp field
    modified_at = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = 'django_dragonpay'
        verbose_name = "Payout"
        verbose_name_plural = "Payouts"

    def __unicode__(self):
        return '%s [%s] %s %s' % (
            self.get_status_display(), self.txn_id, self.amount,
            self.user_id or self.user_name)

    @property
    def is_completed(self):
        # we consider the txn complete if it is success/failed/void
        return self.status in ['S', 'F', 'V']

    def fetch_status(self):
        '''Query DragonPay for the status of this transaction and update the
        database.'''

        from django_dragonpay.api.soap import get_payout_txn_status
        from django_dragonpay.constants import DRAGONPAY_STATUS_CODES

        status = get_payout_txn_status(self.txn_id)

        # update the status
        if status and status != self.status:
            logger.info(
                '[%s] status updated to %s',
                self.txn_id, DRAGONPAY_STATUS_CODES[status])

            self.status = status
            self.save(update_fields=['status'])

    @classmethod
    def create_from_dict(cls, details):
        # Check if we should save to database
        if not DRAGONPAY_SAVE_DATA:
            return

        payouts = []
        if isinstance(details, dict):
            # request is for a single payout
            # it may be registered, or non-registered payout user
            logger.debug(
                'Payout txn [%s] saved to database', details['txn_id'])

            return DragonpayPayout.objects.create(
                txn_id=details['txn_id'],
                user_id=details.get('user_id'),
                user_name=details.get('user_name'),
                processor_id=details.get('processor_id'),
                processor_detail=details.get('processor_detail'),
                email=details.get('email'),
                mobile=details.get('mobile'),
                amount=details['amount'],
                description=details['description'],
                currency=details.get('currency') or 'PHP',
                created_at=details['timestamp'],
            )

        elif isinstance(details, list):
            # request is from MultiplePayout; iterate over the details
            for i, detail in enumerate(details):
                logger.debug(
                    '[%d] Payout txn [%s] saved to database',
                    i, details['txn_id'])

                payouts.append(
                    DragonpayPayout.objects.create(
                        txn_id=detail['txn_id'],
                        user_id=detail['user_id'],
                        amount=detail['amount'],
                        description=detail['description'],
                        currency=details.get('currency') or 'PHP',
                        created_at=detail['timestamp'],
                    ))

            return payouts

        else:
            raise Exception('Invalid details type %s', type(details))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-04-04 10:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DragonpayPayout',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('txn_id', models.CharField(max_length=40)),
                ('user_id', models.CharField(blank=True, max_length=40, null=True)),
                ('user_name', models.CharField(blank=True, max_length=32, null=True)),
                ('processor_id', models.CharField(blank=True, choices=[(b'BDO', b'Banco De Oro'), (b'BPI', b'Bank of the Philippine Islands'), (b'CBC', b'Chinabank'), (b'EWB', b'East West Bank'), (b'LBP', b'Land Bank of the Philippines'), (b'MBTC', b'Metrobank'), (b'PNB', b'Philippine National Bank'), (b'RCBC', b'RCBC'), (b'SBC', b'Security Bank'), (b'UBP', b'Union Bank'), (b'UCPB', b'UCPB'), (b'PSB', b'PS Bank'), (b'CEBL', b'Cebuana Lhuilier'), (b'GCSH', b'GCash'), (b'SMRT', b'Smart Money')], max_length=8, null=True)),
                ('processor_detail', models.CharField(blank=True, max_length=32, null=True)),
                ('email', models.CharField(blank=True, max_length=32, null=True)),
                ('mobile', models.CharField(blank=True, max_length=32, null=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=8)),
                ('description', models.CharField(max_length=128)),
                ('currency', models.CharField(default=b'PHP', max_length=3)),
                ('status', models.CharField(choices=[(b'S', b'Success'), (b'F', b'Failed'), (b'P', b'Pending'), (b'G', b'In progress'), (b'V', b'Voided')], default=b'P', max_length=1)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Payout',
                'verbose_name_plural': 'Payouts',
            },
        ),
        migrations.CreateModel(
            name='DragonpayPayoutUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=32)),
                ('middle_name', models.CharField(max_length=32)),
                ('last_name', models.CharField(max_length=32)),
                ('email', models.CharField(max_length=64)),
                ('birthdate', models.DateField()),
                ('mobile', models.CharField(max_length=24)),
                ('address1', models.CharField(max_length=32)),
                ('address2', models.CharField(max_length=32)),
                ('city', models.CharField(max_length=32)),
                ('country', models.CharField(max_length=16)),
                ('state', models.CharField(max_length=16)),
                ('zip', models.CharField(max_length=8)),
            ],
            options={
                'verbose_name': 'Payout User',
                'verbose_name_plural': 'Payout Users',
            },
        ),
        migrations.CreateModel(
            name='DragonpayTransaction',
            fields=[
                ('id', models.CharField(max_length=40, primary_key=True, serialize=False)),
                ('token', models.CharField(max_length=40)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=8)),
                ('currency', models.CharField(choices=[(b'PHP', b'Philippine Peso'), (b'USD', b'US Dollar')], default=b'PHP', max_length=3)),
                ('description', models.CharField(max_length=128)),
                ('email', models.CharField(max_length=40)),
                ('param1', models.CharField(blank=True, max_length=80, null=True)),
                ('param2', models.CharField(blank=True, max_length=80, null=True)),
                ('status', models.CharField(choices=[(b'S', b'Success'), (b'F', b'Failed'), (b'P', b'Pending'), (b'U', b'Unknown'), (b'R', b'Refund'), (b'K', b'Chargeback'), (b'V', b'Voided'), (b'A', b'Authorized')], default=b'P', max_length=1)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Transaction',
                'verbose_name_plural': 'Transactions',
            },
        ),
    ]



import urllib
import logging

from django.conf.settings import (
    DRAGONPAY_ID, DRAGONPAY_MERCHANT_SECRET, DRAGONPAY_MERCHANT_URL,
    DRAGONPAY_PAY_URL)
from django_dragonpay.utils import get_dragonpay_digest

logger = logging.getLogger('dragonpay.http_get')


def get_txn_url(txn_id, amount, description, email, *params):
    digest_list = [DRAGONPAY_ID, txn_id, amount, description, email]

    payload = {
        'merchantid': DRAGONPAY_ID,
        'txnid': txn_id,
        'amount ': amount,
        'ccy ': 'PHP',
        'description': description,
        'email': email,
        'digest': get_dragonpay_digest(digest_list)
    }

    if len(params) > 2:
        raise Exception(
            'You can only have a maximum of 2 DragonPay request params.')

    # include any params that were included
    for i, param in enumerate(params):
        payload['param%s' % (i + 1)] = param

    return DRAGONPAY_PAY_URL + urllib.urlencode(payload)


def get_txn_status(txn_id):
    payload = {
        'merchantid': DRAGONPAY_ID,
        'merchantpwd': DRAGONPAY_MERCHANT_SECRET,
        'txnid': txn_id,
        'op': 'GETSTATUS'
    }

    response.POST(DRAGONPAY_MERCHANT_URL, data=payload)


def cancel_transaction(txn_id):
    payload = {
        'merchantid': DRAGONPAY_ID,
        'merchantpwd': DRAGONPAY_MERCHANT_SECRET,
        'txnid': txn_id, 'op': 'VOID'
    }

    response.POST(DRAGONPAY_MERCHANT_URL, data=payload)




import logging
from django import forms
from django_dragonpay.utils import decrypt_data
from django_dragonpay import settings as dp_settings

logger = logging.getLogger('dragonpay.forms')


class DragonpayCallbackForm(forms.Form):
    '''Dragonpay form for Name-Value Pair model where data is sent via
    HTTP GET callbacks.'''

    STATUS_CODES = [
        ('S', 'Success'), ('F', 'Failure'), ('P', 'Pending'),
        ('U', 'Unknown'), ('R', 'Refund'), ('K', 'Chargeback'),
        ('V', 'Void'), ('A', 'Authorized')]

    txnid = forms.CharField(max_length=128)
    refno = forms.CharField(max_length=32)
    status = forms.CharField(max_length=1)
    message = forms.CharField(max_length=128)
    digest = forms.CharField(max_length=40)
    param1 = forms.CharField(max_length=128, required=False)
    param2 = forms.CharField(max_length=128, required=False)

    def clean(self):
        '''Custom clean method to verify the message authenticity thru
        the digest.'''

        from django_dragonpay.utils import get_dragonpay_digest

        KEYS = ['txnid', 'refno', 'status', 'message']
        to_digest = ':'.join([self.cleaned_data[key] for key in KEYS])

        digest = get_dragonpay_digest(to_digest)

        # Validate that the message sent is cryptographically valid
        if self.cleaned_data['digest'] != digest:
            logger.error(
                'Request hash [%s] doesnt match caclulated [%s]',
                self.cleaned_data['digest'], digest)

            forms.ValidationError("DragonPay digest doesn't match!")

        # Decrypt params if they are encrypted
        if dp_settings.DRAGONPAY_ENCRYPT_PARAMS:
            for key in ['param1', 'param2']:
                param = self.cleaned_data.get(key)

                if param:
                    self.cleaned_data['param1'] = decrypt_data(param)
                    logger.debug(
                        'Decrypting %s:%s', param, self.cleaned_data['param1'])

        return self.cleaned_data



from django.conf import settings

SECRET_KEY = getattr(settings, 'SECRET_KEY')
# DRAGONPAY API
DRAGONPAY_TEST_MODE = getattr(settings, 'DRAGONPAY_TEST_MODE', False)
DRAGONPAY_ID = getattr(settings, 'DRAGONPAY_ID')
DRAGONPAY_PASSWORD = getattr(settings, 'DRAGONPAY_PASSWORD')
DRAGONPAY_API_KEY = getattr(settings, 'DRAGONPAY_API_KEY')
DRAGONPAY_ENCRYPT_PARAMS = getattr(settings, 'DRAGONPAY_ENCRYPT_PARAMS', False)

# the transaction length, max 40
DRAGONPAY_TXN_LENGTH = getattr(settings, 'DRAGONPAY_TXN_LENGTH', 20)
DRAGONPAY_SAVE_DATA = getattr(settings, 'DRAGONPAY_SAVE_DATA', False)

if DRAGONPAY_TEST_MODE:
    DRAGONPAY_BASE_URL = 'http://test.dragonpay.ph/'
    DRAGONPAY_PAYOUT_BASE_URL = 'http://test.dragonpay.ph/'
else:
    DRAGONPAY_BASE_URL = 'https://gw.dragonpay.ph/'
    DRAGONPAY_PAYOUT_BASE_URL = 'https://live.dragonpay.ph/'

# Other Dragonpay URLs
DRAGONPAY_PAY_URL = DRAGONPAY_BASE_URL + 'Pay.aspx'
DRAGONPAY_MERCHANT_URL = DRAGONPAY_BASE_URL + 'MerchantRequest.aspx'
DRAGONPAY_SOAP_URL = DRAGONPAY_BASE_URL + 'DragonpayWebService/MerchantService.asmx'
DRAGONPAY_PAYOUT_URL = DRAGONPAY_PAYOUT_BASE_URL + 'DragonpayWebService/PayoutService.asmx'



import json
import urllib
import logging
import requests
from lxml import etree
from datetime import datetime

from django.template.loader import render_to_string

from django_dragonpay.models import *
from django_dragonpay.utils import encrypt_data, generate_txn_id
from django_dragonpay import settings as dp_settings
from django_dragonpay.constants import (
    DRAGONPAY_STATUS_CODES, DRAGONPAY_ERROR_CODES,
    DRAGONPAY_PAYOUT_ERROR_CODES)
from django_dragonpay.exceptions import DragonpayException

logger = logging.getLogger('dragonpay')

HEADERS = {'Content-Type': 'text/xml; charset=utf-8'}
# constant context containinig api keys and passwords
CONTEXT = {
    'dp_merchant_id': dp_settings.DRAGONPAY_ID,
    'dp_merchant_apikey': dp_settings.DRAGONPAY_API_KEY,
    'dp_merchant_secret': dp_settings.DRAGONPAY_PASSWORD,
}


def _dragonpay_soap_wrapper(
        webmethod, context={}, xml_name=None, payout=False):
    '''Helper function for SOAP requests to DragonPay Payment Switch (PS).

    webmethod (string) - the DragonPay SOAP web method being called.
    context (dict) - the context to be passed to the XML template
    xml_name (string) - override filename for the XML template file to
        be used, if None, then webmethod.xml will be used.
    payout (boolean) - flag to use the DRAGONPAY_PAYOUT_URL.'''

    # include the configuration constants
    context.update(CONTEXT)
    context['web_method'] = webmethod

    xml = render_to_string(
        'dragonpay_soapxml/%s.xml' % xml_name or webmethod, context)

    headers = {'SOAPAction': "http://api.dragonpay.ph/%s" % webmethod}
    headers.update(HEADERS)

    # check if this is a PAYOUT transaction
    if not payout:
        url = dp_settings.DRAGONPAY_SOAP_URL
    else:
        url = dp_settings.DRAGONPAY_PAYOUT_URL

    logger.debug(
        'Sending SOAP Request to [%s]:\nHEADERS: %s\nXML:\n%s',
        url, headers, xml)
    response = requests.post(url, data=xml, headers=headers)

    try:
        xmltree = etree.fromstring(response.content)
    except etree.XMLSyntaxError:
        # Failed to create an XML object from response.content
        logger.error(
            'Failed to parse reponse as a valid XML: %s', response.content)
        return
    else:
        xml_pretty = etree.tostring(xmltree, pretty_print=True)

    if response.status_code != 200:
        logger.error(
            'Invalid response %s:\n%s', response.status_code, xml_pretty)
        raise DragonpayException(
            '[%d] Dragonpay %s request failed' % (
                response.status_code, webmethod))
    else:
        logger.debug('Success\n%s', xml_pretty)
        return xmltree


def _dragonpay_get_wrapper(webmethod, xml_name=None, context={}, payout=False):
    '''Dragonpay SOAP helper function that returns the result of
    GET WebMethods.'''

    xml_name = xml_name or webmethod
    xmltree = _dragonpay_soap_wrapper(webmethod, context, xml_name, payout)

    # Parse the response XML for the WebMethod specific response
    if xmltree is not None:
        response = xmltree.find(
            './/{http://api.dragonpay.ph/}%(webmethod)sResponse/'
            '{http://api.dragonpay.ph/}%(webmethod)sResult' % {
                'webmethod': webmethod}
        ).text

        return response


def get_txn_url_from_token(token, proc_id=None):
    '''Returns the DragonPay payment URL given a token.'''

    d = {'tokenid': token}
    if proc_id:
        d['procid'] = proc_id

    return dp_settings.DRAGONPAY_PAY_URL + '?' + urllib.urlencode(d)


def get_txn_token_url(amount, description, email, proc_id=None, **params):
    '''Creates a DragonPay transaction and returns the Payment Switch URL.'''

    token = get_txn_token(amount, description, email, **params)

    if token:
        return get_txn_url_from_token(token[1], proc_id)


def get_txn_token(amount, description, email, txn_id=None, **params):
    '''Requests for a new DragonPay transaction and returns its txn_id and token.
    If not txn_id is passed, this method will generate one.

    return (tuple) - (txn_id, token)'''

    logger.debug(
        'get_txn_token %s %s %s %s', email, amount, description, params)

    txn_id = txn_id or generate_txn_id()
    context = {
        'txn_id': txn_id, 'amount': amount, 'email': email,
        'description': 'description'}

    logger.debug('params %s', params)
    # include the params in the context
    for key, value in params.iteritems():
        if dp_settings.DRAGONPAY_ENCRYPT_PARAMS:
            # Encrypt the params to obfuscate the payload
            logger.debug('Encrypting %s', value)
            value = encrypt_data(value)

        context[key] = value

    logger.debug('get_txn_token payload: %s', context)
    token = _dragonpay_get_wrapper('GetTxnToken', context=context)

    context['token'] = token
    context.update(params)      # include the raw params back to context
    DragonpayTransaction.create_from_dict(context)

    # check if the response token is an error code
    if len(token) < 4:
        logger.error(
            '[%s] Dragonpay Error: %s',
            token, DRAGONPAY_ERROR_CODES[token])

        return

    logger.debug('[%s] token %s for %s PhP %s', txn_id, token, email, amount)

    return txn_id, token


def get_txn_status(txn_id):
    '''Fetches the transaction status given a transaction id.'''

    context = {'txn_id': txn_id}
    txn_status = _dragonpay_get_wrapper('GetTxnStatus', 'webmethod', context)

    logger.debug(
        '[%s] txn status %s', txn_id, DRAGONPAY_STATUS_CODES[txn_status])

    return txn_status


def cancel_transaction(txn_id):
    '''Cancels the transaction given a transaction id. Returns True if the
    cancellation succeeds.'''

    context = {'txn_id': txn_id}
    status = _dragonpay_get_wrapper('CancelTransaction', context=context)

    if status == '0':
        logger.debug('[%s] Txn cancellation success', txn_id)
        return True

    else:
        logger.debug('[%s] Txn cancellation failed: %s', txn_id, status)


def get_txn_ref_no(txn_id):
    '''Fetches the reference number of a transaction.'''

    context = {'txn_id': txn_id}

    refno = _dragonpay_get_wrapper('GetTxnRefNo', 'webmethod', context)
    logger.debug('[%s] reference no: %s', txn_id, refno)
    return refno


def get_available_processors(amount):
    context = {'amount': amount}
    context.update(CONTEXT)

    xml = render_to_string('dragonpay_soapxml/webmethod.xml', context=context)

    return _dragonpay_soap_wrapper(xml, success=_success, error=_error)


def get_email_instructions(refno):
    response = requests.get(
        dp_settings.DRAGONPAY_BASE_URL + 'Bank/GetEmailInstruction.aspx',
        params={'refno': refno, 'format': 'json'})

    if response.status_code == 200:
        return json.loads(response.content)

    else:
        logger.error(
            'Error in getting email instructions: %s %s',
            response.status_code, response.content)


# PAYOUT RELATED SOAP METHODS
def _get_payout_data(webmethod, xml_name=None):
    '''Helper function for fetching data related to Payout.'''

    xmltree = _dragonpay_soap_wrapper(
        webmethod, xml_name=xml_name or 'GetPayoutData', payout=True)

    xmltree = xmltree.find(
        './/{http://api.dragonpay.ph/}%(webmethod)sResponse/'
        '{http://api.dragonpay.ph/}%(webmethod)sResult' % {
            'webmethod': webmethod}
    )
    data = []

    # convert the xmltree to dict
    for result in xmltree:
        subdata = {}
        for detail in result:
            subdata[detail.tag[detail.tag.index('}') + 1:]] = detail.text

        data.append(subdata)

    return data


def get_countries():
    '''Fetches data for the GetCountries Payout WebMethod.'''
    return _get_payout_data('GetCountries')


def get_processors():
    '''Fetches data for the GetProcessors Payout WebMethod.'''
    return _get_payout_data('GetProcessors')


def get_payout_txn_status(txn_id):
    '''Fetches the status of a payout transaction.'''
    context = {'txn_id': txn_id}

    txn_status = _dragonpay_get_wrapper(
        'GetTxnStatus', xml_name='GetPayoutTxnStatus',
        context=context, payout=True)

    logger.debug(
        '[%s] txn status %s',
        txn_id, DRAGONPAY_STATUS_CODES[txn_status])

    return txn_status


def modify_payout_channel():
    return _dragonpay_get_wrapper('ModifyPayoutChannel', payout=True)


def register_payout_user(user_details):
    '''Register a user for payout. The registered user can be given a payout
    using the request_payout method using his registered user_id.

    user_details (dict) - the user details of the user.
        see payout_context_keys for the list of fields that are required.'''

    payout_context_keys = {
        'address1', 'address2', 'birthdate', 'city', 'country', 'email',
        'first_name', 'last_name', 'middle_name', 'mobile', 'state', 'zip'}
    # Check that the given context contains the required fields
    if not set(user_details.keys()) == payout_context_keys:
        logger.debug(
            'Keys [%s] are missing from the RegisterPayoutUser context',
            ', '.join(set(user_details.keys()) - payout_context_keys))

        raise Exception('RegisterPayoutUser context invalid contents')

    return _dragonpay_get_wrapper(
        'RegisterPayoutUser', context=user_details, payout=True)


def request_multiple_payouts(payout_details):
    '''Requests for multiple payouts. This method and API only works for
    registered payout users.

    payout_details should contain the keys:
        txn_id, user_id, amount, currency, description'''

    if not isinstance(payout_details, list):
        raise Exception('payout_details must be a list')

    DragonpayPayout.create_from_dict(payout_details)

    rmp = 'RequestMultiplePayouts'
    return _get_payout_data(rmp, rmp)


def request_payout(txn_id, user_id, amount, description, currency=None):
    '''Request a payout to a registered user.'''

    context = {
        'txn_id': txn_id, 'user_id': user_id, 'amount': amount,
        'description': description, 'currency': currency}

    response_code = _dragonpay_soap_wrapper(
        'RequestPayout', context=context, payout=True)

    if response_code == '0':
        # save the dragonpay payout transaction to the database
        DragonpayPayout.create_from_dict(context)
    else:
        logger.error(
            '[%s] %s', response_code,
            DRAGONPAY_PAYOUT_ERROR_CODES[response_code])

    return response_code


def request_payout_ex(
        user_name, amount, description, proc_id,
        proc_detail, email, mobile, currency=None):
    '''Request for a one-time payout.

    user_name - the name of the individual that will receive the payout.
    amount - the amount to be given as payout.
    description - a short description for this transaction.
    proc_id - the processor id; see list of processors via get_processors
        method.
    proc_detail - the processor detail.
    email - the email of the individual that will receive the payout.
    mobile - the mobile number of the individual.
    currency - the currency to be used, defaults to PHP when None.

    Note: Payout transaction fees are not subtracted to the amount but
        will be shouldered by the merchant. So if we send amount=500,
        with a processor whose fee is PhP 15, PhP 515 will be deducted
        from our DragonPay account.'''

    txn_id = generate_txn_id()
    context = {
        'txn_id': txn_id,
        'user_name': user_name,
        'amount': amount,
        'currency': currency,
        'description': description,
        'processor_id': proc_id,
        'processor_detail': proc_detail,
        'timestamp': datetime.now(),
        'email': email,
        'mobile': mobile,
    }

    response_code = _dragonpay_get_wrapper(
        'RequestPayoutEx', context=context, payout=True)

    if response_code == '0':
        # save the dragonpay payout transaction to the database
        DragonpayPayout.create_from_dict(context)
    else:
        logger.error(
            '[%s] %s', response_code,
            DRAGONPAY_PAYOUT_ERROR_CODES[response_code])

    return response_code, txn_id



import base64
import random
import string
from Crypto import Random
from Crypto.Cipher import AES
from hashlib import sha1, sha256

from django_dragonpay import settings


def get_dragonpay_digest(str_or_list):
    '''The DragonPay digest is a concatination of the values passed and
    the DRAGONPAY_PASSWORD, joined by a colon(":").'''

    if isinstance(str_or_list, list) or isinstance(str_or_list, tuple):
        # The given parameter is list or tuple, convert it to a valid
        # digestible message by joining it with colons
        str_or_list = ':'.join(str_or_list)

    # Append the MERCHANT_PASSWORD to the string and return the sha1 digest
    return sha1(
        str_or_list + ':' + settings.DRAGONPAY_PASSWORD).hexdigest()


# http://stackoverflow.com/a/21928790
class AESCipher(object):
    '''Cipher helper class used to encrypt and decrypt data via AES CBC.'''

    def __init__(self):
        self.bs = 32
        self.key = sha256(settings.SECRET_KEY.encode()).digest()

    def encrypt(self, raw):
        raw = self._pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return base64.b64encode(iv + cipher.encrypt(raw))

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(
            cipher.decrypt(enc[AES.block_size:])).decode('utf-8')

    def _pad(self, s):
        return s + (
            self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s) - 1:])]


def encrypt_data(message):
    cipher = AESCipher()
    return cipher.encrypt(message)


def decrypt_data(message):
    cipher = AESCipher()
    return cipher.decrypt(message)


def generate_txn_id():
    '''Generates a random transaction id.'''

    return ''.join([random.choice(
        string.hexdigits) for i in range(settings.DRAGONPAY_TXN_LENGTH)])



from setuptools import setup

setup(
    name='django_dragonpay',
    version='0.1.2',
    description='DragonPay plugin for Django',
    author='Ivan Dominic Baguio',
    author_email='baguio.ivan@gmail.com',
    url='https://github.com/ibaguio/django_dragonpay',
    packages=[
        'django_dragonpay', 'django_dragonpay.api',
        'django_dragonpay.migrations'],
    install_requires=[
        'requests>=2.11.1',
        'Django>=1.8.0',
        'lxml==3.7.3',
        'pycrypto>=2.6.1',
    ],
    keywords='django dragonpay payment',
    license='MIT',
    include_package_data=True,
)

__author__ = 'Ivan Dominic Baguio'



# Dragonpay status codes
# See Appendix 3 of Dragonpay API documentation
DRAGONPAY_STATUS_CODES = {
    'S': 'Success',
    'F': 'Failed',
    'P': 'Pending',
    'U': 'Unknown',
    'R': 'Refund',
    'K': 'Chargeback',
    'V': 'Voided',
    'A': 'Authorized',
    'G': 'In progress',
}

# Dragonpay error codes.
# See Appendix 2 of Dragonpay API documentation
DRAGONPAY_ERROR_CODES = {
    '000': 'Success',
    '101': 'Invalid payment gateway id',
    '102': 'Incorrect secret key',
    '103': 'Invalid reference number',
    '104': 'Unauthorized access',
    '105': 'Invalid token',
    '106': 'Currency not supported',
    '107': 'Transaction cancelled',
    '108': 'Insufficient funds',
    '109': 'Transaction limit exceeded',
    '110': 'Error in operation',
    '111': 'Invalid parameters',
    '201': 'Invalid Merchant Id',
    '202': 'Invalid Merchant Password'
}


DRAGONPAY_PAYOUT_ERROR_CODES = {
    '0': 'Successfully created payout request',
    '-1': 'Invalid credentials or apiKey',
    '-2': '(reserved)',
    '-3': '(reserved)',
    '-4': 'Unable to create payout transaction (internal error)',
    '-5': 'Invalid account no / details',
    '-6': 'Invalid pre-dated run date',
    '-7': 'Amount exceeds limit for payout channel',
    '-8': 'A payout has been previously requested for the same merchant txn id'
}

# Dragonpay Paymemnt method FILTERS
ONLINE_BANKING = 1       # Online banking
OTC_BANK = 2             # Over-the-Counter Banking and ATM
OTC_NON_BANK = 4         # Over-the-Counter non-Bank
# 8 (unused)
# 16 (reserved internally)
PAYPAL = 32              # PayPal
CREDIT_CARDS = 64        # Credit Cards
MOBILE = 128             # Mobile (Gcash)
INTERNATIONAL_OTC = 256  # International OTC
